package com.docmosis.example;


import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.net.URISyntaxException;

import com.docmosis.SystemManager;
import com.docmosis.document.DocumentProcessor;
import com.docmosis.document.ProcessingException;
import com.docmosis.document.converter.ConversionException;
import com.docmosis.document.converter.ConversionFormat;
import com.docmosis.document.converter.ConversionInstruction;
import com.docmosis.document.converter.NoConvertersRunningException;
import com.docmosis.document.converter.pool.ConverterPoolException;
import com.docmosis.template.TemplateStoreException;
import com.docmosis.template.population.Cleanable;
import com.docmosis.template.population.DataProviderBuilder;
import com.docmosis.template.store.DropStoreHelper;
import com.docmosis.template.store.TemplateContext;
import com.docmosis.template.store.TemplateIdentifier;
import com.docmosis.template.store.TemplateStoreFactory;

/**
 * Example code demonstrating some of the ways that Docmosis can be used to merge documents.
 */
public class Example5
{

	private static void printUsageAndExit()
	{
		System.err.println("usage: java " + Example5.class.getName() + " [-h] <n> ");
		System.err.println("  -h  : print this usage");
		System.err.println("  <n> : create n documents each distinctly numbered");
		System.err.println("Each document render takes the mainTemplate.doc template and the sampleData.txt " +
				"file and creates result.pdf.");
		System.exit(1);
	}
	
	/**
	 * Run the program.
	 * @param args - see usage.
	 * @throws ProcessingException 
	 * @throws TemplateStoreException 
	 * @throws ConverterPoolException 
	 * @throws ConversionException 
	 * @throws IOException
	 * @throws NoConvertersRunningException
	 */
	public static void main(String[] args)
	throws IOException, ConversionException, ConverterPoolException, TemplateStoreException,
	ProcessingException, URISyntaxException, NoConvertersRunningException
	{
		if (args.length > 1 || (args.length > 0 && "-h".equals(args[0]))) {
			printUsageAndExit();
			System.exit(1);
		}
		
		int runs = 1;
		if (args.length > 0) {
			try {
				runs = Integer.parseInt(args[0]);
			} catch(NumberFormatException e) {
				System.err.println("The argument must be an integer between 0 and 1000");
				System.exit(1);
			}
		}
		
		if (runs > 1000 || runs < 1) {
			System.err.println("The argument must be an integer between 0 and 1000");
			System.exit(1);
		}

		// initialise the system based on configuration
    	SystemManager.initialise();
    	
		DataProviderBuilder dpb = new DataProviderBuilder();
    	try {
    		
    		// load the templates into a context "myTemplates"
    		TemplateContext context = new TemplateContext("myTemplates");
    		new DropStoreHelper(TemplateStoreFactory.getStore()).process(new File("templates"), context);
    		
    		// read the data file using a '|' delimiter
    		dpb.addFile(new File("sampleData.txt"), '|');
    		
    		// choose the template
    		TemplateIdentifier templateId = new TemplateIdentifier("mainTemplate.doc", context);
    		
    		// choose the output format
    		ConversionInstruction instruction = new ConversionInstruction();
    		instruction.setConversionFormats(new ConversionFormat[] {
    				ConversionFormat.FORMAT_PDF
    		});
    		// since we are re-using the data provider, we don't want it cleaned up automatically
    		instruction.setCleanupDataProvider(false);
    		
    		System.out.println("About to generate " + runs + " document(s).");
    		long start = System.currentTimeMillis();
    		for(int i=0; i < runs; i++) {
            	FileOutputStream outStream = null;
    			try {
	        		// create the destination file.
    				final String fileName = (runs == 1) ? "result.pdf" : "result" + i + ".pdf"; 
	        		outStream = new FileOutputStream(fileName);
	        		
	        		// render the document
	    			DocumentProcessor.renderDoc(templateId, dpb.getDataProvider(), instruction, outStream);
    			} finally {
    	    		if (outStream != null) {
    	    			outStream.close();
    	    			outStream = null;
    	    		}
    			}
    		}
    		long end = System.currentTimeMillis();
    		
    		System.out.println(runs + " document(s) produced in " + (end - start) + " millis (avg = " + ((end-start)/runs) + ").");
    		
    	} finally {
    		if (dpb.getDataProvider() instanceof Cleanable) {
    			((Cleanable)dpb.getDataProvider()).cleanup();
    		}
    		SystemManager.release();
    	}
	}
}
